module m_phase1

  implicit none

  integer, save, dimension(5) :: pxfact, pyfact
  integer, save, dimension(5) :: pxsfact, pysfact
  integer, allocatable, dimension(:), save :: specskx, specsky
  complex, allocatable, dimension(:,:), save :: omegaxbr, omegaybr, omegaxbi, omegaybi
  complex, allocatable, dimension(:,:), save :: omegaxnr, omegaynr, omegaxni, omegayni
  complex, allocatable, dimension(:,:), save :: omegaxbrs, omegaybrs, omegaxbis, omegaybis
  complex, allocatable, dimension(:,:), save :: omegaxnrs, omegaynrs, omegaxnis, omegaynis

contains

  subroutine phase1( nx, ny, spgix, spgiy )

  use Math_Const
  use ffts

  implicit none

  integer, intent(in) :: nx, ny, spgix, spgiy
  integer :: i, j, ngx, ngy

  ngx=nx/spgix
  ngy=ny/spgiy

  call rotate_array_f()
  call prim_calc( nx, pxfact(1:4), pxfact(5) )
  call prim_calc( ny, pyfact(1:4), pyfact(5) )
  call prim_calc( ngx, pxsfact(1:4), pxsfact(5) )
  call prim_calc( ngy, pysfact(1:4), pysfact(5) )

  allocate(omegaxnr(0:nx-1,0:nx-1))
  allocate(omegaynr(0:ny-1,0:ny-1))
  allocate(omegaxni(0:nx-1,0:nx-1))
  allocate(omegayni(0:ny-1,0:ny-1))
  allocate(omegaxbr(0:pxfact(5)-1,0:pxfact(5)-1))
  allocate(omegaybr(0:pyfact(5)-1,0:pyfact(5)-1))
  allocate(omegaxbi(0:pxfact(5)-1,0:pxfact(5)-1))
  allocate(omegaybi(0:pyfact(5)-1,0:pyfact(5)-1))
  allocate(omegaxnrs(0:ngx-1,0:ngx-1))
  allocate(omegaynrs(0:ngy-1,0:ngy-1))
  allocate(omegaxnis(0:ngx-1,0:ngx-1))
  allocate(omegaynis(0:ngy-1,0:ngy-1))
  allocate(omegaxbrs(0:pxsfact(5)-1,0:pxsfact(5)-1))
  allocate(omegaybrs(0:pysfact(5)-1,0:pysfact(5)-1))
  allocate(omegaxbis(0:pxsfact(5)-1,0:pxsfact(5)-1))
  allocate(omegaybis(0:pysfact(5)-1,0:pysfact(5)-1))
  allocate(specskx(ngx))
  allocate(specsky(ngy))

  call rotate_calc( nx, 'r', pxfact,  &
  &               omegaxbr(0:pxfact(5)-1,0:pxfact(5)-1),  &
  &               omegaxnr(0:nx-1,0:nx-1) )
  call rotate_calc( nx, 'i', pxfact,  &
  &               omegaxbi(0:pxfact(5)-1,0:pxfact(5)-1),  &
  &               omegaxni(0:nx-1,0:nx-1) )
  call rotate_calc( ny, 'r', pyfact,  &
  &               omegaybr(0:pyfact(5)-1,0:pyfact(5)-1),  &
  &               omegaynr(0:ny-1,0:ny-1) )
  call rotate_calc( ny, 'i', pyfact,  &
  &               omegaybi(0:pyfact(5)-1,0:pyfact(5)-1),  &
  &               omegayni(0:ny-1,0:ny-1) )

  call rotate_calc( ngx, 'r', pxsfact,  &
  &               omegaxbrs(0:pxsfact(5)-1,0:pxsfact(5)-1),  &
  &               omegaxnrs(0:ngx-1,0:ngx-1) )
  call rotate_calc( ngx, 'i', pxsfact,  &
  &               omegaxbis(0:pxsfact(5)-1,0:pxsfact(5)-1),  &
  &               omegaxnis(0:ngx-1,0:ngx-1) )
  call rotate_calc( ngy, 'r', pysfact,  &
  &               omegaybrs(0:pysfact(5)-1,0:pysfact(5)-1),  &
  &               omegaynrs(0:ngy-1,0:ngy-1) )
  call rotate_calc( ngy, 'i', pysfact,  &
  &               omegaybis(0:pysfact(5)-1,0:pysfact(5)-1),  &
  &               omegaynis(0:ngy-1,0:ngy-1) )

  do i=1,ngx
     specskx(i)=(i-1)*spgix+1
  end do
  do j=1,ngy
     specsky(j)=(j-1)*spgiy+1
  end do

  end subroutine phase1

end module m_phase1



module m_phase2

  implicit none

contains

  subroutine phase2( nx, ny, ival1, oval0, oval1, oval2, spgix, spgiy, dt )

  use Math_Const
  use ffts
  use m_phase1

  implicit none

  integer, intent(in) :: nx, ny, spgix, spgiy
  real, dimension(nx,ny), intent(in) :: ival1
  real, dimension(nx,ny), intent(out) :: oval0, oval1, oval2
  real, dimension(2), intent(out) :: dt

  integer :: i, j, ngx, ngy
  real :: ts1, ts2, ts3, ts4
  complex, dimension(nx,ny) :: sp1, sp2, spi1, spi2
  complex, dimension(nx/spgix,ny/spgiy) :: cpval1, cpval2

  ngx=nx/spgix
  ngy=ny/spgiy

  do j=1,ngy
     do i=1,ngx
        cpval1(i,j)=ival1(specskx(i),specsky(j))
     end do
  end do

  call cpu_time(ts1)

  call ffttp_2d( ngx, ngy, cpval1(1:ngx,1:ngy), cpval2(1:ngx,1:ngy),  &
  &              'r', 'o', prim_factx=pxsfact, prim_facty=pysfact,  &
  &              omegax_fix=omegaxbrs, omegaxn_fix=omegaxnrs,  &
  &              omegay_fix=omegaybrs, omegayn_fix=omegaynrs )

  call cpu_time(ts2)

  sp1=0.0
  sp2=0.0

!-- WN0
  sp1(1,1)=cpval2(1,1)

!-- WN0
  sp2(1,1)=cpval2(1,1)

!-- WNK1
  sp2(2,1)=cpval2(2,1)
  sp2(nx,1)=cpval2(nx,1)

!-- WNL1
  sp2(1,2)=cpval2(1,2)
  sp2(1,ny)=cpval2(1,ny)

! ʲ, ɬפʤ
!  call ffttp_2d( nx, ny, cpval2(1:nx,1:ny), cpval1(1:nx,1:ny),  &
!  &              'i', 'o', prim_factx=pxfact, prim_facty=pyfact,  &
!  &              omegax_fix=omegaxbi, omegaxn_fix=omegaxni,  &
!  &              omegay_fix=omegaybi, omegayn_fix=omegayni )

!  call ffttp_2d( nx, ny, sp1(1:nx,1:ny), spi1(1:nx,1:ny),  &
!  &              'i', 'o', prim_factx=pxfact, prim_facty=pyfact,  &
!  &              omegax_fix=omegaxbi, omegaxn_fix=omegaxni,  &
!  &              omegay_fix=omegaybi, omegayn_fix=omegayni )

  call cpu_time(ts3)

  call ffttp_2d( nx, ny, sp2(1:nx,1:ny), spi2(1:nx,1:ny),  &
  &              'i', 'o', prim_factx=pxfact, prim_facty=pyfact,  &
  &              omegax_fix=omegaxbi, omegaxn_fix=omegaxni,  &
  &              omegay_fix=omegaybi, omegayn_fix=omegayni )

  call cpu_time(ts4)

  do j=1,ny
     do i=1,nx
        oval0(i,j)=real(ival1(i,j))
        oval1(i,j)=real(spi1(i,j))
        oval2(i,j)=real(spi2(i,j))
     end do
  end do

  dt(1)=ts2-ts1
  dt(2)=ts4-ts3

  end subroutine phase2

end module m_phase2

program phase8

!-- phase7 Ʊ, Ѵǡְǹ®.
!-- ǡΥե, ؤνϤϰڹԤʤ.
!-- phase5 Ǥϥ֥롼 phase1 Υ롼Ƥ֤Ȥ FFT 
!-- ž׻Ƥ, ׻Ψ.
!-- ܥץǤϲž׻롼 phase1,
!-- ʳ FFT 롼ˤĤƤ phase2 Ȥ̾Υ롼
!-- ѰդƷ׻.
!-- phase5 ץȤбط
!-- phase5 : subroutine phase1 = phase6 : subrutines phase1 and phase2
!-- ꥸʥǡκ,  root ץǹԤ.

  use m_phase1
  use m_phase2
!  use basis
  use mpi
!  use dcl
!  use dcl_automatic

  implicit none

  integer, parameter :: nx=200, ny=300, nz=50
  integer, parameter :: npx=2, npy=2
  integer, parameter :: spgix=5, spgiy=5
  integer, parameter :: root=0

  integer :: i, j, k, icounter, ip, jp
  integer :: MY_RANK, PETOT, IERROR   ! For MPI variables
  real :: x(nx), y(ny)
  real, allocatable, dimension(:,:) :: valp1
  real, allocatable, dimension(:) :: totsp1, totsp2
  real, dimension(nx,ny,nz) :: ival1
  real, dimension(nx,ny) :: oval0, oval1, oval2
  real :: t1, t2, t3
  real :: totdt(2), dt(2)
  real :: tffts, tffte, tperiod

!-- Initializing MPI

  call MPI_INIT( IERROR )

!-- Getting total process number and oneself process ID.

  call MPI_COMM_RANK( MPI_COMM_WORLD, MY_RANK, IERROR )
  call MPI_COMM_SIZE( MPI_COMM_WORLD, PETOT, IERROR )

  if(MY_RANK==root)then
     call cpu_time( t1 )
     tperiod=0.0
     dt=0.0
     totdt=0.0
  end if

  allocate(totsp1(nx*ny))
  allocate(totsp2((nx/npx)*(ny/npy)))
  allocate(valp1(nx/npx,ny/npy))

  if(MY_RANK==root)then

     do k=1,nz
        do j=1,ny
           do i=1,nx
              ival1(i,j,k)=exp(-(((real(i)-0.5*real(nx))/real(nx))**2  &
  &                             +((real(j)-0.5*real(ny))/real(ny))**2  &
  &                             +((real(k)-0.5*real(nz))/real(nz))**2))
           end do
        end do
     end do

     x=(/((real(i)/real(nx)),i=1,nx)/)
     y=(/((real(j)/real(ny)),j=1,ny)/)

!-- Calculate the rotate arrays

     call phase1( nx, ny, spgix, spgiy )

!------------- DCL
!     call SGISET('IFONT',2 )
!     CALL GLRSET( 'RMISS', -999.0 )
!     CALL GLLSET( 'LMISS', .TRUE. )
!     call UZFACT(0.75)

!     call color_setting( 80, (/0.0, 1.0/), min_tab=15999,  &
!  &                      max_tab=95999, col_min=10, col_max=99 )

!     call DclOpenGraphics(1)
      call cpu_time(t3)

  end if

  do k=1,nz

!-- rearranging from data array to comm array

     if(MY_RANK==root)then
        icounter=0

        do jp=1,npy
           do ip=1,npx
              do j=1,ny/npy
                 do i=1,nx/npx
                    icounter=icounter+1
                    totsp1(icounter)=ival1((ip-1)*nx/npx+i,(jp-1)*ny/npy+j,k)
                 end do
              end do
           end do
        end do
     end if

     call MPI_BARRIER( MPI_COMM_WORLD, IERROR )

!-- splitting the whole data to partial data

     call MPI_SCATTER( totsp1, (nx/npx)*(ny/npy), MPI_REAL,  &
  &                    totsp2, (nx/npx)*(ny/npy), MPI_REAL,  &
  &                    root, MPI_COMM_WORLD, IERROR )

     call MPI_BARRIER( MPI_COMM_WORLD, IERROR )

!-- rearranging from comm array to data array

     if(MY_RANK==root)then
        totsp1=0.0
     end if

     icounter=0

     do j=1,ny/npy
        do i=1,nx/npx
           icounter=icounter+1
           valp1(i,j)=totsp2(icounter)
        end do
     end do

!     open(unit=10+MY_RANK,  &
!  &       file='pe'//trim(adjustl( i2c_convert(MY_RANK,forma='(i4.4)') ))//'.bin',  &
!  &    recl=4*(nx/npx)*(ny/npy),access='direct',status='replace')

!       write(10+MY_RANK,rec=1) ((valp1(i,j),i=1,nx/npx),j=1,ny/npy)

!     close(unit=10+MY_RANK)

     call MPI_BARRIER( MPI_COMM_WORLD, IERROR )

!     valp1=0.0

!     call MPI_BARRIER( MPI_COMM_WORLD, IERROR )

!     open(unit=1000+MY_RANK,  &
!  &       file='pe'//trim(adjustl( i2c_convert(MY_RANK,forma='(i4.4)') ))//'.bin',  &
!  &       recl=4*(nx/npx)*(ny/npy),access='direct',status='old')

!        read(1000+MY_RANK,rec=1) ((valp1(i,j),i=1,nx/npx),j=1,ny/npy)

!     close(unit=1000+MY_RANK)

     icounter=0

     do j=1,ny/npy
        do i=1,nx/npx
           icounter=icounter+1
           totsp2(icounter)=valp1(i,j)
        end do
     end do

     call MPI_BARRIER( MPI_COMM_WORLD, IERROR )

!-- gathering the partial data to the whole data

     call MPI_GATHER( totsp2, (nx/npx)*(ny/npy), MPI_REAL,  &
  &                   totsp1, (nx/npx)*(ny/npy), MPI_REAL,  &
  &                   root, MPI_COMM_WORLD, IERROR )

     call MPI_BARRIER( MPI_COMM_WORLD, IERROR )

     if(MY_RANK==root)then
!-- rearranging from comm array to data array

        icounter=0

        do jp=1,npy
           do ip=1,npx
              do j=1,ny/npy
                 do i=1,nx/npx
                    icounter=icounter+1
                    ival1((ip-1)*nx/npx+i,(jp-1)*ny/npy+j,k)=totsp1(icounter)
                 end do
              end do
           end do
        end do

        call cpu_time( tffts )
        call phase2( nx, ny, ival1(:,:,k), oval0, oval1, oval2, spgix, spgiy, dt )
        call cpu_time( tffte )
        tperiod=tperiod+(tffte-tffts)

        totdt(1)=totdt(1)+dt(1)
        totdt(2)=totdt(2)+dt(2)

!        call Dcl_2D_cont_shade( 'Full(z='//  &
!  &                             trim(adjustl(i2c_convert(k,forma='(i3.3)')))//')',  &
!  &                             x, y, ival1(:,:,k), oval0,  &
!  &                             (/0.0, 1.0/), (/0.0, 1.0e-4/),  &
!  &                             (/'X (km)', 'Y (km)'/),  &
!  &                             (/'(f6.1)', '(f6.1)'/),  &
!  &                             c_num=(/10, 80/), no_tone=.true. )

!        call Dcl_2D_cont_shade( 'WN0(z='//  &
!  &                             trim(adjustl(i2c_convert(k,forma='(i3.3)')))//')',  &
!  &                             x, y, ival1(:,:,k), oval1,  &
!  &                             (/0.0, 1.0/), (/0.0, 1.0e-4/),  &
!  &                             (/'X (km)', 'Y (km)'/),  &
!  &                             (/'(f6.1)', '(f6.1)'/),  &
!  &                             c_num=(/10, 80/), no_tone=.true. )

!        call Dcl_2D_cont_shade( 'WN1(z='//  &
!  &                             trim(adjustl(i2c_convert(k,forma='(i3.3)')))//')',  &
!  &                             x, y, ival1(:,:,k), oval2,  &
!  &                             (/0.0, 1.0/), (/0.0, 1.0e-4/),  &
!  &                             (/'X (km)', 'Y (km)'/),  &
!  &                             (/'(f6.1)', '(f6.1)'/),  &
!  &                             c_num=(/10, 80/), no_tone=.true. )

     end if

     call MPI_BARRIER( MPI_COMM_WORLD, IERROR )

  end do

!  if(MY_RANK==root)then

!     call DclCloseGraphics

!  end if

  if(MY_RANK==root)then
     call cpu_time( t2 )
     write(*,*) "Result : ", t2-t1, "[s]."
     write(*,*) "Loop   : ", t2-t3, "[s]."
     write(*,*) "rotate : ", t3-t1, "[s]."
     write(*,*) "FFT    : ", tperiod, "[s]."
     write(*,*) "RFFT   : ", totdt(1), "[s]."
     write(*,*) "IFFT   : ", totdt(2), "[s]."
  end if

!-- finishing MPI process

  call MPI_FINALIZE( IERROR )

end program phase8
