module force_solv
! ׻⥸塼
  use Derivation
  use max_min
  use Statistics
  use Math_Const
  use Phys_Const
  use val_define
  use read_namelist
  use val_alloc
  use val_coord
  use real_initialize
  use sub_calc

contains

subroutine force()

  implicit none

  integer :: i, j

  !-- ν

  call real_init( force_omega )
  call real_init( force_buoy )

  !-- ۤη׻

  call grad_2d( x, y, omega_old, dodx, dody, undeff=undef )
  call grad_2d( x, y, buoy_old, dbdx, dbdy, undeff=undef )
  call grad_2d( x, y, psi_old, dpdx, dpdy, undeff=undef )

!$omp parallel default(shared)
!$omp do schedule(dynamic) private(i,j)
  do j=1,ny
     do i=1,nx
        if(calc_flag(i,j).eqv..true.)then
           pdoy(i,j)=psi_old(i,j)*dody(i,j)
           pdox(i,j)=psi_old(i,j)*dodx(i,j)
           odpy(i,j)=omega_old(i,j)*dpdy(i,j)
           odpx(i,j)=omega_old(i,j)*dpdx(i,j)
        end if
     end do
  end do
!$omp end do
!$omp end parallel

!$omp parallel default(shared)
!$omp do schedule(dynamic) private(i)
  do i=1,nx
     call grad_1d( y, pdox(i,:), dpdoxdy(i,:), undef=undef )
     call grad_1d( y, odpx(i,:), dodpxdy(i,:), undef=undef )
     call laplacian_1d( y, omega_old(i,:), do2dy2(i,:), undef=undef )
     call laplacian_1d( y, buoy_old(i,:), db2dy2(i,:), undef=undef )
  end do
!$omp end do
!$omp end parallel

!$omp parallel default(shared)
!$omp do schedule(dynamic) private(j)
  do j=1,ny
     call grad_1d( x, pdoy(:,j), dpdoydx(:,j), undef=undef )
     call grad_1d( x, odpy(:,j), dodpydx(:,j), undef=undef )
     call laplacian_1d( x, omega_old(:,j), do2dx2(:,j), undef=undef )
     call laplacian_1d( x, buoy_old(:,j), db2dx2(:,j), undef=undef )
  end do
!$omp end do
!$omp end parallel

  !-- γƹ׻, 

!$omp parallel default(shared)
!$omp do schedule(dynamic) private(i,j)

  do j=1,ny
     do i=1,nx
        if(calc_flag(i,j).eqv..true.)then
           AJ(i,j,1)=dpdx(i,j)*dody(i,j)-dpdy(i,j)*dodx(i,j)
           AJ(i,j,2)=dpdoydx(i,j)-dpdoxdy(i,j)
           AJ(i,j,3)=dodpxdy(i,j)-dodpydx(i,j)
           ADV(i,j)=(AJ(i,j,1)+AJ(i,j,2)+AJ(i,j,3))/3.0
           DIFF(i,j)=nu*(do2dx2(i,j)+do2dy2(i,j))
           force_omega(i,j)=ADV(i,j)+DIFF(i,j)-dbdx(i,j)
           force_buoy(i,j)=dpdx(i,j)*dbdy(i,j)-dpdy(i,j)*dbdx(i,j)  &
  &                       +nu*(db2dx2(i,j)+db2dy2(i,j))
        end if
     end do
  end do

!$omp end do
!$omp end parallel

end subroutine

end module
